﻿using MediatR;
using Sharing.Notifications;
using Sharing.Province.Dtos.HuiJu;

namespace Sharing.Province.Handlers
{
    public class OrderCreatedHandler : INotificationHandler<OrderCreatedNotification>
    {
        private readonly ProvinceClient _provinceClient;

        public OrderCreatedHandler(ProvinceClient provinceClient)
        {
            _provinceClient = provinceClient;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(OrderCreatedNotification notification, CancellationToken cancellationToken)
        {
            var order = notification.DataOrder;

            var pusher = PusherProvider.CreatePusher(_provinceClient);


            var data = new SubmitCaseInfoData
            {
                CaseInfos = new List<CaseInfo>
                {
                    new CaseInfo
                    {
                        OrderNo = order.OrderNo,
                        OrderTitle = order.OrderId
                    }
                }
            };
            var request = new SubmitCaseInfoRequest();
            request.SetData(data);


            await pusher.PushOrderCreatedAsync(request, cancellationToken);
        }
    }
}