﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using IdentityModel.Client;
using Sharing.Province.Dtos;
using Sharing.Province.Extensions;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Sharing.Province
{
    public class ProvinceClient : IScopeDependency, ISelfDependency
    {
        private static readonly string KeyToken = "KeyToken";
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IHttpInvoker _httpInvoker;
        private readonly ITypedCache<TokenInfo> _cacheToken;

        public ProvinceClient(
            IChannelConfigurationManager channelConfigurationManager,
            IHttpInvoker httpInvoker,
            ITypedCache<TokenInfo> cacheToken)
        {
            _channelConfigurationManager = channelConfigurationManager;
            _httpInvoker = httpInvoker;
            _cacheToken = cacheToken;
        }

        public async Task<TResponse?> RequestAsync<TRequest, TResponse>(TRequest request, CancellationToken cancellationToken)
            where TRequest : IProvinceRequest, new()
        {
            var token = _cacheToken.GetOrSet(KeyToken,
                d => GetTokenAsync(cancellationToken).GetAwaiter().GetResult(),
                TimeSpan.FromMinutes(28));
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            request.BuildClientInfo(configProvince.ClientId, configProvince.ClientSecret);

            //todo token失效重新获取token再次请求，返回参数缺少该状态，暂无法处理

            return await _httpInvoker.RequestAsync<TRequest, TResponse>(request,
                d => d.SetHttpClient(configProvince.Address, token?.AccessToken ?? string.Empty),
                cancellationToken);
        }

        private async Task<TokenInfo> GetTokenAsync(CancellationToken cancellationToken)
        {
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            var baseAddress = configProvince.Address;
            if (!baseAddress.EndsWith('/'))
                baseAddress += "/";
            var request = new ClientCredentialsTokenRequest
            {
                ClientId = configProvince.ClientId,
                ClientSecret = configProvince.ClientSecret,
                Address = $"{baseAddress}oauth2/token",
                Headers =
                {
                    Accept =
                    {
                        MediaTypeWithQualityHeaderValue.Parse("application/x-www-form-urlencoded")
                    }
                }

            };
            var tokenResponse = await _httpInvoker.GetTokenAsync(request, cancellationToken);
            if (tokenResponse is null)
                throw new UserFriendlyException("获取token请求失败");
            if (tokenResponse.IsError)
                throw new UserFriendlyException("获取token请求失败");

            var tokenInfo = System.Text.Json.JsonSerializer.Deserialize<TokenRsp>(tokenResponse.Raw);
            if (tokenInfo?.TokenInfo is null)
                throw new UserFriendlyException("token解析失败");
            return tokenInfo.TokenInfo;
        }
    }
}
